<?php
if (!defined('ABSPATH')) exit;

class ICP_Manager {
    public function __construct() {

    }
    public function init() {

    }

    public function store(ICP_Countdown $countdown) {
        global $icp;
        $countdown->id=intval($countdown->id);
        $array=$icp->Options->getArrayCountdowns();
        if($countdown->id<=0) {
            $max=0;
            foreach($array as $k=>$v) {
                if($k>$max) {
                    $max=$k;
                }
            }
            $countdown->id=($max+1);
        }
        $array[$countdown->id]=$countdown;
        $icp->Options->setArrayCountdowns($array);
    }
    public function query($id=FALSE) {
        global $icp;
        $result=$icp->Options->getArrayCountdowns();
        if($id!==FALSE) {
            $result=(isset($result[$id]) ? $result[$id] : FALSE);
        }
        return $result;
    }
    public function get($id, $new=FALSE) {
        global $icp;
        $result=$this->query($id);
        /* @var $result ICP_Countdown */
        if($result===FALSE && $new) {
            $result=$icp->Dao->Utils->newDomainClass('Countdown');
            $result->evergreen=1;
            $result->expireSlotsIn=$icp->Utils->formatTimer(3*86400);
            $result->expireDateIn=$icp->Utils->formatTimer(3*86400);
            $result->expirationDate=time()+(7*86400);
            $result->availableSlots=50;
            $result->automaticResetDays=90;
            $result->labelsSlots='spots,spot';

            $result->color=ICP_CountdownConstants::COLOR_BLACK;
            $result->type=ICP_CountdownConstants::TYPE_DATE;
            $result->detect=ICP_CountdownConstants::DETECT_COOKIE;

            $result->digitsFontSize=80;
            $result->labelsFontSize=20;

            $result->labelsDays='days,day';
            $result->labelsHours='hours,hour';
            $result->labelsMinutes='minutes,minute';
            $result->labelsSeconds='seconds,second';
        }
        return $result;
    }
    public function delete($ids) {
        global $icp;
        $ids=$icp->Utils->toArray($ids);
        $array=$icp->Options->getArrayCountdowns();
        foreach($ids as $id) {
            unset($array[$id]);
            $icp->Options->removeIpAddressFirstSeen($id);
            $icp->Options->removeUsernameFirstSeen($id);
        }
        $icp->Options->setArrayCountdowns($array);
        return TRUE;
    }
    public function copy($ids) {
        global $icp;
        $ids=$icp->Utils->toArray($ids);
        if(count($ids)==0) {
            return FALSE;
        }

        $array=$icp->Options->getArrayCountdowns();
        $result=FALSE;
        $e=FALSE;
        foreach($ids as $id) {
            $result=TRUE;
            if(isset($array[$id])) {
                $e=clone($array[$id]);
                /* @var $e ICP_Countdown */
                $e->id=0;
                $e->name='Copy of '.$e->name;
                $this->store($e);
            }
        }
        if($e!==FALSE) {
            $icp->Ui->redirectEdit($e->id);
        }
        return $result;
    }
    public function getDetectStrategy($detect) {
        $result=FALSE;
        switch ($detect) {
            case ICP_CountdownConstants::DETECT_COOKIE:
                $result=new ICP_CookieDetect();
                break;
            case ICP_CountdownConstants::DETECT_COOKIE_IP_ADDRESS:
                $result=new ICP_CookieIpAddressDetect();
                break;
            case ICP_CountdownConstants::DETECT_IP_ADDRESS:
                $result=new ICP_IpAddressDetect();
                break;
            case ICP_CountdownConstants::DETECT_USERNAME:
                $result=new ICP_UsernameDetect();
                break;
        }
        return $result;
    }

    public function hasIPOnlyCounters() {
        global $icp;
        global $pagenow;

        if ($pagenow === 'options-general.php') {
            // start sessions in the admin tabs
            return false;
        }

        $countdowns = $icp->Options->getArrayCountdowns();
        foreach ($countdowns as $value) {
            if ($value->detect != ICP_CountdownConstants::DETECT_IP_ADDRESS) {
                return false;
            }
        }
        return true;
    }
}