<?php
if (!defined('ABSPATH')) exit;

class ICP_CountdownUi {
    var $classCountdown='';
    var $classDigit='';
    var $classLabel='';
    var $classBoxDigit='';
    var $classBoxSeparator='';

    var $firstSeenValues=array();

    public function __construct() {
    }
    private function drawDigit(ICP_Countdown $countdown, $labels) {
        $words = explode(',', $labels);
        $words[0] = __($words[0], 'intelly-countdown-pro');
        $words[1] = __($words[1], 'intelly-countdown-pro');
        $labels = implode(',', $words)
        ?>
        <div class="<?php echo $this->classBoxDigit ?>">
            <div class="<?php echo $this->classDigit ?>" style="font-size: <?php echo $countdown->digitsFontSize?>px;">0</div>
            <div class="<?php echo $this->classLabel ?>" style="font-size: <?php echo $countdown->labelsFontSize?>px;" icp-labels="<?php echo $labels?>">LBL</div>
        </div>
    <?php }
    private function drawSeparator(ICP_Countdown $countdown) { ?>
        <div class="<?php echo $this->classBoxSeparator ?>">
            <div class="<?php echo $this->classDigit ?>" style="font-size: <?php echo $countdown->digitsFontSize?>px;">:</div>
            <div></div>
        </div>
    <?php }
    private function generateClasses() {
        if($this->classCountdown!='') {
            return;
        }

        $this->classCountdown='icp-countdown';
        $this->classDigit='icp-digit';
        $this->classLabel='icp-label';
        $this->classBoxDigit='icp-box-digit';
        $this->classBoxSeparator='icp-box-separator';
        /*if(FALSE && ICP_FREE_VERSION) {
            $this->classCountdown=$this->generateMd5($this->classCountdown);
            $this->classDigit=$this->generateMd5($this->classDigit);
            $this->classLabel=$this->generateMd5($this->classLabel);
            $this->classBoxDigit=$this->generateMd5($this->classBoxDigit);
            $this->classBoxSeparator=$this->generateMd5($this->classBoxSeparator);
        }*/
    }
    private function generateMd5($name) {
        $prefix='a';
        $name.='-t'.time();
        $result=$prefix.md5($name);
        return $result;
    }
    public function draw($countdown) {
        global $icp;
        if($countdown===FALSE) {
            return;
        }

        /* user details */
        $current_user = wp_get_current_user();
        $roles = $current_user->roles;
        $role = array_shift($roles);

        /* @var $countdown ICP_Countdown */
        $expireIn='';
        switch ($countdown->type) {
            case ICP_CountdownConstants::TYPE_DATE:
                $expireIn=$countdown->expireDateIn;
                break;
            case ICP_CountdownConstants::TYPE_SLOTS:
                $expireIn=$countdown->expireSlotsIn;
                break;
        }

        $restart=0;
        $immediateRestart=0;
        $expiration=$icp->Utils->parseDateToTime($countdown->expirationDate);
        $redirectExpirationDate = $icp->Utils->parseDateToTime($countdown->redirectExpirationDate);

        if ($countdown->type === ICP_CountdownConstants::TYPE_DAILY) {
            $expiration='';
            $exclude = [];
            $exclude[0] = $countdown->excludeSun;
            $exclude[1] = $countdown->excludeMon;
            $exclude[2] = $countdown->excludeTue;
            $exclude[3] = $countdown->excludeWed;
            $exclude[4] = $countdown->excludeThu;
            $exclude[5] = $countdown->excludeFri;
            $exclude[6] = $countdown->excludeSat;

            $day = date('d');
            $month = date('m');
            $year = date('Y');
            $target = mktime(0, 0, 0, $month, $day, $year) + $icp->Utils->parseTimer($countdown->expirationTime);
            $now = time();

            $skipDays = 0;
            $today = date('w'); // day of week (0 to 6)
            if ($target < $now) {
                $target = $target + 60 * 60 * 24;
                $skipDays++;
            }

            while (true) {
                $n = $today+$skipDays;
                if ($n > 6) {
                    $n -= 7;
                }
                if ($exclude[$n]) {
                    $target = $target + 60 * 60 * 24;
                    $skipDays++;
                    if ($skipDays > 6) {
                        break;
                    }
                } else {
                    break;
                }
            }

            $expireIn = $icp->Utils->formatTimer($target);
        } else if ($countdown->evergreen) {
            $expiration='';
            $redirectExpirationDate='';

            $detect=$icp->Manager->getDetectStrategy($countdown->detect);
            $firstSeen=$detect->getFirstSeen($countdown);
            if($firstSeen!==FALSE) {
                $expire=$icp->Utils->parseTimer($expireIn);
                $reset=intval($countdown->automaticResetDays)*86400;
                if ($countdown->redirectExpirationDays > 0) {
                    $redirectExpirationDate = $firstSeen+$expire+$countdown->redirectExpirationDays*86400;
                }
                if($reset>0 && $expire>0) {
                    $now=time();
                    $target=$firstSeen+$expire+$reset;
                    if($now>$target) {
                        $restart=1;
                    }
                }
                $immediateRestart = $countdown->immediateRestart;
            }
            if($restart==0 && $firstSeen!==FALSE) {
                $this->firstSeenValues[$countdown->id]=$firstSeen;
            }
        }

        $redirectUri = ($role !== 'administrator' ) ? $countdown->redirectUri : '';

        $this->generateClasses();
        echo '<div class="'.$this->classCountdown.'" style="display:none; color:'.$countdown->color.';" icp-expire="'.$expireIn.'" icp-type="'.$countdown->type.'" icp-slots="'.$countdown->availableSlots.'" icp-reset="'.$countdown->automaticResetDays.'" icp-redirect="'.$redirectUri.'" icp-redirect-expiration="'.$redirectExpirationDate.'" icp-id="'.$countdown->id.'" icp-restart="'.$restart.'" icp-immediate-restart="'.intval($immediateRestart).'" icp-eg="'.($icp->Utils->isTrue($countdown->evergreen) ? 1 : 0).'" icp-expiration="'.$expiration.'">';
        switch ($countdown->type) {
            case ICP_CountdownConstants::TYPE_DATE:
            case ICP_CountdownConstants::TYPE_DAILY:
                $this->drawDigit($countdown, $countdown->labelsDays);
                $this->drawSeparator($countdown);
                $this->drawDigit($countdown, $countdown->labelsHours);
                $this->drawSeparator($countdown);
                $this->drawDigit($countdown, $countdown->labelsMinutes);
                $this->drawSeparator($countdown);
                $this->drawDigit($countdown, $countdown->labelsSeconds);
                break;
            case ICP_CountdownConstants::TYPE_SLOTS:
                $this->drawDigit($countdown, $countdown->labelsSlots);
                break;
        }
        echo '</div>';
        /*?>
        <style>
            .<?php echo $this->classCountdown?>.icp-<?php echo $countdown->id ?> {
                color: <?php echo $countdown->color?>;
            }
            .<?php echo $this->classCountdown?>.icp-<?php echo $countdown->id ?> .<?php echo $this->classDigit?> {
                font-size: <?php echo $countdown->digitsFontSize?>px;
            }
            .<?php echo $this->classCountdown?>.icp-<?php echo $countdown->id ?> .<?php echo $this->classLabel?> {
                font-size: <?php echo $countdown->labelsFontSize?>px;
            }
        </style>
        <?php*/
    }
    public function script() {
        global $icp;
        //$offset=(new DateTime('now'))->getOffset();
        ?>
        <script>
            <?php /*var clientTimezoneOffset=-(new Date().getTimezoneOffset());
            ICP.setCookie('GTM', clientTimezoneOffset);
            ICP.setCookie('SGTM', -<?php echo $offset?>);*/?>

            var dt=0;
            <?php
            foreach($this->firstSeenValues as $id=>$time) {
                if($time===FALSE) {
                    continue;
                }
                ?>
                dt=moment.unix(<?php echo $time?>).toDate();
                ICP.setDateCookie('<?php echo $id?>_FirstSeen', dt);
            <?php } ?>

            var ICP_FIRST_SEEN={};
            var ecTimer=null;
            function enCountdownSetLabel($text, $labels, index, value, label) {
                $text=jQuery($text);
                if($text.text()!=value) {
                    $text.text(value);
                }

                if($labels.length>index) {
                    var $self=jQuery($labels[index]);
                    var labels=ICP.attr($self, 'icp-labels', label);
                    labels=labels.split(',');
                    if(labels.length==1) {
                        labels.push(labels[0]);
                    }
                    value=parseInt(value);
                    label=(value===1 ? labels[1] : labels[0]);
                    if($self.text()!=label) {
                        $self.text(label);
                    }
                }
            }
            function ecCountdownsLoop() {
                jQuery('.<?php echo $this->classCountdown ?>').each(function(i,v) {
                    var $self=jQuery(this);
                    var classLabel='<?php echo $this->classLabel?>';
                    var classDigit='<?php echo $this->classDigit?>';
                    var classBoxSeparator='<?php echo $this->classBoxSeparator?>';

                    var id=ICP.attr($self, 'icp-id');
                    var redirectUri=ICP.attr($self, 'icp-redirect');
                    var redirectExpiration=ICP.attr($self, 'icp-redirect-expiration');
                    redirectExpiration=moment.unix(redirectExpiration).toDate();
                    var type=ICP.attr($self, 'icp-type');
                    var slots=ICP.attr($self, 'icp-slots');
                    var expire=ICP.attr($self, 'icp-expire');
                    var reset=parseInt(ICP.attr($self, 'icp-reset'));
                    var restart=parseInt(ICP.attr($self, 'icp-restart'));
                    var immediateRestart=parseInt(ICP.attr($self, 'icp-immediate-restart'));

                    var eg=parseInt(ICP.attr($self, 'icp-eg'));
                    var expiration=parseInt(ICP.attr($self, 'icp-expiration'));
                    expiration=moment.unix(expiration).toDate();

                    var now=new Date();
                    var time=0;
                    var firstSeen=0;

                    if (type === 'DAILY') {
                        time = ICP.parseTimer(expire);
                        time -= parseInt(now.getTime()/1000);
                    } else if (eg==0) {
                        time=parseInt((expiration.getTime()-now.getTime())/1000);
                    } else {
                        firstSeen = ICP_FIRST_SEEN[id];

                        if(firstSeen==null || firstSeen===undefined) {
                            firstSeen=ICP.getDateCookie(id+'_FirstSeen');
                            //console.log('firstSeen=%s', firstSeen);
                            if(firstSeen==null || restart!=0) {
                                firstSeen=new Date();
                                ICP.setDateCookie(id+'_FirstSeen', firstSeen);
                            }
                        }

                        time=ICP.parseTimer(expire);
                        time-=parseInt((now.getTime()-firstSeen.getTime())/1000);

                        if (immediateRestart != 0) {
                            if (time <= 0) {
                                if (reset > 0) {
                                    var resetTime = reset * 86400;
                                    if (time + resetTime < 0) {
                                        firstSeen = new Date();
                                        ICP.setDateCookie(id + '_FirstSeen', firstSeen);
                                    }
                                } else {
                                    firstSeen = new Date();
                                    ICP.setDateCookie(id + '_FirstSeen', firstSeen);
                                }
                            }
                        }

                        ICP_FIRST_SEEN[id] = firstSeen;
                    }

                    if (time <= 0) {
                        time=0;
                        var isRedirectExpired = false;
                        if (redirectExpiration > 0) {
                            if (now.getTime() > redirectExpiration.getTime()) {
                                isRedirectExpired = true;
                            }
                        }
                        if (redirectUri != '' && !isRedirectExpired) {
                            location.href = redirectUri;
                        }
                    }

                    var $digits=$self.find('.'+classDigit);
                    var $labels=$self.find('.'+classLabel);
                    var $separators=$self.find('.'+classBoxSeparator);

                    var dt1=0;
                    var dt2=0;
                    switch (type) {
                        case 'SLOTS':
                            if(time>0) {
                                if(eg==0) {
                                    time=slots;
                                } else {
                                    dt1=(firstSeen.getTime()/1000);
                                    dt2=dt1+ICP.parseTimer(expire);
                                    now=(now.getTime()/1000);
                                    time=parseInt((slots*(dt2-now))/(dt2-dt1));
                                }
                            }
                            enCountdownSetLabel($digits[0], $labels, 0, time, 'Slots');
                            break;
                        case 'DAILY':
                        case 'DATE':
                            time=ICP.formatTimer(time);
                            time=time.split(':');
                            var days=time[0];
                            var hours=time[1];
                            var minutes=time[2];
                            var seconds=time[3];
                            if(parseInt(days)<=0) {
                                jQuery($digits[0]).hide();
                                jQuery($labels[0]).hide();
                                jQuery($separators[0]).hide();
                            }
                            enCountdownSetLabel($digits[0], $labels, 0, days, 'Days');
                            enCountdownSetLabel($digits[2], $labels, 1, hours, 'Hours');
                            enCountdownSetLabel($digits[4], $labels, 2, minutes, 'Minutes');
                            enCountdownSetLabel($digits[6], $labels, 3, seconds, 'Seconds');
                            break;
                    }
                    $self.show();
                });
            }

            ecTimer=setInterval(function() {
                ecCountdownsLoop();
            }, 1000);
            ecCountdownsLoop();
        </script>
    <?php }
    public function style() { ?>
        <style>
            .<?php echo $this->classCountdown?> {
                display: table;
                margin: 0px;
                padding: 0px;
                width: 50%;
                margin: auto;
            }
            .<?php echo $this->classCountdown?> .<?php echo $this->classDigit?> {
                text-align: center;
                margin: 0px;
                padding: 0px;
                line-height: normal;
            }
            .<?php echo $this->classCountdown?> .<?php echo $this->classLabel?> {
                text-align: center;
                margin: 0px;
                padding: 0px;
                line-height: normal;
            }
            .<?php echo $this->classCountdown?> .<?php echo $this->classBoxDigit?> {
                display: table-cell;
                margin: 0px;
                padding: 0px;
            }
            .<?php echo $this->classCountdown?> .<?php echo $this->classBoxSeparator?> {
                display: table-cell;
                margin: 0px;
                padding: 0px;
                padding-left: 5px;
                padding-right: 5px;
            }
        </style>
    <?php }
}